
from dataclasses import dataclass, field
from typing import TypeVar, Set

from chess.offset import LINEAR
from chess.path import Path
from chess.piece import Piece
from chess.piece_meta import PieceMeta


class RookMeta(PieceMeta):
    name: str = 'Rook'
    letter: str = 'R'
    value: int = 5
    symbol: str = '♜'
    html_decimal: str = '&#9820;'
    html_hex: str = '&#x265C;'

    move_paths: Set[Path] = {Path(offset=offset, max_steps=None) for offset in LINEAR}
    capture_paths: Set[Path] = {Path(offset=offset, max_steps=None) for offset in LINEAR}


@dataclass(frozen=True)
class Rook(Piece):
    meta: RookMeta = RookMeta







from dataclasses import dataclass

@dataclass(frozen=True)
class Offset:
    dx: int = 0
    dy: int = 0

    def __add__(self, other):
        if isinstance(other, Offset):
            return Offset(dx=self.dx + other.dx, dy=self.dy + other.dy)
        else:
            return Offset(dx=self.dx + int(other), dy=self.dy + int(other))

    def __sub__(self, other):
        if isinstance(other, Offset):
            return Offset(dx=self.dx - other.dx, dy=self.dy - other.dy)
        else:
            return Offset(dx=self.dx - int(other), dy=self.dy - int(other))

    def __mul__(self, other):
        if isinstance(other, Offset):
            return Offset(dx=self.dx * other.dx, dy=self.dy * other.dy)
        else:
            return Offset(dx=self.dx * int(other), dy=self.dy * int(other))

    def is_linear(self) -> bool:
        return self.dx == 0 or self.dy == 0

    def is_diagonal(self) -> bool:
        return self.dx != 0 and self.dy != 0


UP = Offset(dx=0, dy=-1)
DOWN = Offset(dx=0, dy=1)
LEFT = Offset(dx=-1, dy=0)
RIGHT = Offset(dx=1, dy=0)

UP_LEFT = UP + LEFT
UP_RIGHT = UP + RIGHT
DOWN_LEFT = DOWN + LEFT
DOWN_RIGHT = DOWN + RIGHT

VERTICAL = {UP, DOWN}
HORIZONTAL = {LEFT, RIGHT}
LINEAR = VERTICAL | HORIZONTAL
DIAGONAL = {UP_LEFT, UP_RIGHT, DOWN_LEFT, DOWN_RIGHT}
OMNI = LINEAR | DIAGONAL




from dataclasses import dataclass, field
from typing import TypeVar, Set

from chess.move import Move
from chess.offset import Offset, UP, RIGHT, DOWN, LEFT, UP_RIGHT, DOWN_RIGHT, DOWN_LEFT, UP_LEFT
from chess.path import Path
from chess.piece import Piece
from chess.piece_meta import PieceMeta


class KnightMeta(PieceMeta):
    name: str = 'Knight'
    letter: str = 'N'
    value: int = 3
    symbol: str = '\u2658'
    html_decimal: str = '&#9822;'
    html_hex: str = '&#x2658;'
    offsets: Set[Offset] = {
        UP * 2 + RIGHT,
        UP * 2 + LEFT,
        DOWN * 2 + RIGHT,
        DOWN * 2 + LEFT,
        RIGHT * 2 + UP,
        RIGHT * 2 + DOWN,
        LEFT * 2 + UP,
        LEFT * 2 + DOWN,
    }
    move_paths: Set[Path] = {Path(offset=offset, max_steps=1) for offset in offsets}
    capture_paths: Set[Path] = {Path(offset=offset, max_steps=1) for offset in offsets}


@dataclass(frozen=True)
class Knight(Piece):
    meta: KnightMeta = KnightMeta






from typing import Set

from chess.path import Path


class PieceMeta:
    name: str
    letter: str
    value: int
    symbol: str
    html_decimal: str
    html_hex: str

    move_paths: Set[Path] = set()
    capture_paths: Set[Path] = set()
    en_passant_paths: Set[Path] = set()
    castle_paths: Set[Path] = set()





from dataclasses import dataclass
from typing import Set

from chess.offset import VERTICAL, DIAGONAL
from chess.path import Path
from chess.piece import Piece
from chess.piece_meta import PieceMeta


class PawnMeta(PieceMeta):
    name: str = 'Pawn'
    letter: str = 'P'
    value: int = 1
    symbol: str = '\u2659'
    html_decimal: str = '&#9817;'
    html_hex: str = '&#x2659;'

    move_paths: Set[Path] = {Path(offset=offset, max_steps=1) for offset in VERTICAL}
    capture_paths: Set[Path] = {Path(offset=offset, max_steps=1) for offset in DIAGONAL}
    en_passant_paths: Set[Path] = {Path(offset=offset, max_steps=1) for offset in DIAGONAL}


@dataclass(frozen=True)
class Pawn(Piece):
    meta: PawnMeta = PawnMeta






from dataclasses import dataclass

from chess.color import Color
from chess.position import Position
from chess.side import Side


@dataclass(frozen=True)
class CastleRight:
    color: Color
    rook_origin: Position
    rook_destination: Position
    king_origin: Position
    king_destination: Position







from dataclasses import dataclass

from .offset import Offset


@dataclass(frozen=True)
class Position:
    file: int  # 1-indexed
    rank: int  # 1-indexed

    def __post_init__(self):
        if self.file <= 0:
            raise ValueError(f'File must be a positive integer: {self.file}')
        if self.rank <= 0:
            raise ValueError(f'Rank must be a positive integer: {self.rank}')

    def offset(self, offset: Offset) -> 'Position':
        return Position(file=self.file + offset.dx, rank=self.rank + offset.dy)







from dataclasses import dataclass, field
from typing import TypeVar, Set

from chess.offset import OMNI
from chess.path import Path
from chess.piece import Piece
from chess.piece_meta import PieceMeta


class QueenMeta(PieceMeta):
    name: str = 'Queen'
    letter: str = 'Q'
    value: int = 9
    symbol: str = '♛'
    html_decimal: str = '&#9819;'
    html_hex: str = '&#x265B;'

    move_paths: Set[Path] = {Path(offset=offset, max_steps=None) for offset in OMNI}
    capture_paths: Set[Path] = {Path(offset=offset, max_steps=None) for offset in OMNI}


@dataclass(frozen=True)
class Queen(Piece):
    meta: QueenMeta = QueenMeta







from dataclasses import dataclass, field
from typing import Set, Type

from chess.position import Position
from chess.piece import Piece


@dataclass(frozen=True)
class Move:
    piece: Piece
    origin: Position
    destination: Position
    captures: Set[Piece] = field(default_factory=set)
    promotion: Type[Piece] | None = None




from dataclasses import dataclass, field, replace, make_dataclass
from typing import Set, Type

from chess.color import Color
from chess.piece_meta import PieceMeta
from chess.position import Position


@dataclass(frozen=True)
class Piece:
    position: Position
    color: Color
    meta: PieceMeta = PieceMeta
    has_moved: bool = False

    def move(self, position: Position) -> 'Piece':
        return replace(self, position=position, has_moved=True)

    def promote(self, promotion: Type['Piece']) -> 'Piece':
        return replace(self, meta=promotion.meta)






from typing import Set, Dict, Type

from chess.bishop import Bishop
from chess.castle_right import CastleRight
from chess.color import Color
from chess.knight import Knight
from chess.move import Move
from chess.offset import Offset
from chess.path import Path
from chess.pawn import Pawn
from chess.piece import Piece
from chess.position import Position
from chess.queen import Queen
from chess.rook import Rook
from chess.size import Size


class Board:
    size: Size = Size(width=8, height=8)
    color_promotion_positions: Dict[Color, Set[Position]] = {
        Color.WHITE: {Position(file=file, rank=8) for file in range(1, 9)},
        Color.BLACK: {Position(file=file, rank=1) for file in range(1, 9)},
    }
    allowed_promotions: Set[Type[Piece]] = {
        Knight,
        Bishop,
        Rook,
        Queen,
    }

    def __init__(self,
                 pieces: Set[Piece],
                 castling_rights: Set[CastleRight],
                 en_passant_target_position: Position = None,
                 half_move_draw_clock: int = 0,
                 full_move_number: int = 0):
        self.pieces: Set[Piece] = pieces
        self.castling_rights: Set[CastleRight] = castling_rights
        self.en_passant_target_position: Position = en_passant_target_position
        self.half_move_draw_clock: int = half_move_draw_clock
        self.full_move_number: int = full_move_number

    def move(self, piece: Piece, destination: Position):
        self._validate_destination_is_empty(destination=destination)
        self._validate_in_bounds(position=destination)

        self.pieces.remove(piece)
        self.pieces.add(piece.move(destination))

    def _validate_destination_is_empty(self, destination: Position):
        if self.get_piece(destination) is not None:
            raise ValueError(f'Piece already at {destination}')

    def _validate_in_bounds(self, position: Position):
        if not self.in_bounds(position):
            raise ValueError(f'Position {position} is out of bounds')

    def promote(self, piece: Piece, promotion: Type[Piece]):
        self._validate_is_allowed_promotion(promotion=promotion)
        self.pieces.remove(piece)
        self.pieces.add(piece.promote(promotion=promotion))

    def _validate_is_allowed_promotion(self, promotion: Type[Piece]):
        if promotion not in self.allowed_promotions:
            raise ValueError(f'Invalid promotion: {promotion}')

    def get_colored_pieces(self, color: Color) -> Set[Piece]:
        return {piece for piece in self.pieces if piece.color == color}

    def get_piece(self, position: Position) -> Piece | None:
        for piece in self.pieces:
            if piece.position == position:
                return piece
        return None

    def get_max_steps(self, position: Position, offset: Offset) -> int:
        max_dx = max(self.size.width - position.file, position.file) // offset.dx
        max_dy = max(self.size.height - position.rank, position.rank) // offset.dy
        return min(max_dx, max_dy)

    def is_promotion_position(self, color: Color, position: Position) -> bool:
        return position in self.color_promotion_positions[color]

    def in_bounds(self, position: Position) -> bool:
        return self._in_width_bounds(position) and self._in_height_bounds(position)

    def _in_width_bounds(self, position: Position) -> bool:
        return 1 <= position.file <= self.size.width

    def _in_height_bounds(self, position: Position) -> bool:
        return 1 <= position.rank <= self.size.height
















# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['chess']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': '56kyle-pychess',
    'version': '0.3.0',
    'description': 'A python chess engine',
    'long_description': '\n# Pychess\n\n[![Status][status badge]][status badge]\n[![Tests][github actions badge]][github actions page]\n[![Codecov][codecov badge]][codecov page]\n[![Python Version][python version badge]][github page]\n[![License][license badge]][license]\n\n[code of conduct]: https://github.com/56kyle/pychess/blob/master/CODE_OF_CONDUCT.md\n[codecov badge]: https://codecov.io/gh/56kyle/pychess/branch/master/graph/badge.svg?token=0QDENTNTN7\n[codecov page]: https://app.codecov.io/gh/56kyle/pychess/branch/master\n[contributor covenant badge]: https://img.shields.io/badge/Contributor%20Covenant-2.1-4baaaa.svg\n[github actions badge]: https://github.com/56kyle/pychess/workflows/Tests/badge.svg\n[github actions page]: https://github.com/56kyle/pychess/actions?workflow=Tests\n[github page]: https://github.com/56kyle/pychess\n[license badge]: https://img.shields.io/github/license/56kyle/pychess\n[license]: https://opensource.org/licenses/MIT\n[python version badge]: https://img.shields.io/pypi/pyversions/56kyle-pychess\n[status badge]: https://img.shields.io/pypi/status/56kyle-pychess\n\nA chess library written in Python.\n\n\n[Pychess](#Pychess)\n    - [Description](#Description)\n    - [Installation](#Installation)\n    - [Usage](#Usage)\n        - [Game](#Game)\n        - [Board](#Board)\n        - [Move](#Move)\n        - [Piece](#Piece)\n        - [Player](#Player)\n        - [Square](#Square)\n    - [Contributing](#Contributing)\n    - [License](#License)\n\n\n## Installation\n    \n    ```bash\n    # Install from PyPI\n    pip install 56kyle-pychess\n\n    # Install from poetry\n    poetry add 56kyle-pychess\n    ```\n\n## Description\nThe main purpose of this library is to try and practice constantly improving the quality of a codebase instead of allowing complexity to grow with time.\n\nI was mainly inspired by the books "Clean Code" and "Clean Coder" both written by Robert C. Martin. Most of the code in this library is written with the principles of clean code in mind.\n\n## Usage\n### Game\n    TODO\n### Board\n    TODO\n### Move\n    TODO\n### Piece\n    TODO\n### Player\n    TODO\n### Square\n    TODO\n    \n\n\n    \n\n\n\n',
    'author': 'kyle',
    'author_email': '56kyleoliver@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/56kyle/pychess',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)




from typing import Set

from chess.offset import DIAGONAL
from chess.path import Path
from chess.piece import Piece
from chess.piece_meta import PieceMeta


class BishopMeta(PieceMeta):
    name: str = 'Bishop'
    letter: str = 'B'
    value: int = 3
    symbol: str = '♝'
    html_decimal: str = '&#9821;'
    html_hex: str = '&#x265D;'

    move_paths: Set[Path] = {Path(offset=offset, max_steps=None) for offset in DIAGONAL}
    capture_paths: Set[Path] = {Path(offset=offset, max_steps=None) for offset in DIAGONAL}


class Bishop(Piece):
    meta: BishopMeta = BishopMeta





from dataclasses import dataclass
from typing import Set

from chess.move import Move
from chess.offset import HORIZONTAL, OMNI
from chess.path import Path
from chess.piece import Piece
from chess.piece_meta import PieceMeta


class KingMeta(PieceMeta):
    name: str = 'King'
    letter: str = 'K'
    value: int = 0
    symbol: str = '♚'
    html_decimal: str = '&#9818;'
    html_hex: str = '&#x265A;'

    move_paths: Set[Path] = {Path(offset=offset, max_steps=1) for offset in OMNI}
    capture_paths: Set[Path] = {Path(offset=offset, max_steps=1) for offset in OMNI}
    castle_paths: Set[Path] = {Path(offset=offset*2, max_steps=1) for offset in HORIZONTAL} \
                              | {Path(offset=offset*3, max_steps=1) for offset in HORIZONTAL}


@dataclass(frozen=True)
class King(Piece):
    meta: KingMeta = KingMeta






from dataclasses import dataclass
from typing import Set

from chess.offset import Offset
from chess.size import Size


@dataclass(frozen=True)
class Path:
    offset: Offset
    max_steps: int | None = None



